package mods.immibis.microblocks;


import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

import mods.immibis.core.MainThreadTaskQueue;
import mods.immibis.core.api.multipart.ICoverSystem;
import mods.immibis.core.api.multipart.IMultipartTile;
import mods.immibis.core.api.net.IPacket;
import net.minecraft.client.Minecraft;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.world.World;
import cpw.mods.fml.relauncher.Side;
import cpw.mods.fml.relauncher.SideOnly;

public class PacketMicroblockContainerDescription implements IPacket {
	
	public int x, y, z;
	public byte[] data;

	@Override
	public byte getID() {
		return MicroblockSystem.PKT_S2C_MICROBLOCK_CONTAINER_DESCRIPTION;
	}

	@Override
	public String getChannel() {
		return MicroblockSystem.CHANNEL;
	}

	@Override
	public void read(DataInputStream in) throws IOException {
		x = in.readInt();
		y = in.readInt();
		z = in.readInt();
		int len = in.readShort();
		data = new byte[len];
		in.read(data);
	}

	@Override
	public void write(DataOutputStream out) throws IOException {
		out.writeInt(x);
		out.writeInt(y);
		out.writeInt(z);
		out.writeShort(data.length);
		out.write(data);
	}
	
	@SideOnly(Side.CLIENT)
	private boolean tryApply() {
		World w = Minecraft.getMinecraft().theWorld;
		TileEntity te = w.getTileEntity(x, y, z);
		if(te instanceof IMultipartTile) {
			ICoverSystem ci = ((IMultipartTile)te).getCoverSystem();
			if(ci instanceof MicroblockCoverSystem) {
				((MicroblockCoverSystem)ci).readDescriptionBytes(data, 0);
				return true;
			} else
				return false;
		} else
			return false;
	}

	@Override
	@SideOnly(Side.CLIENT)
	public void onReceived(EntityPlayer source) {
		//System.out.println(Thread.currentThread().getName());
		if(!tryApply())
			MainThreadTaskQueue.enqueue(new Runnable() {
				private int retries = 0;
				@Override
				public void run() {
					if(!tryApply())
						if(retries < 20) {
							retries++;
							MainThreadTaskQueue.enqueue(this, Side.CLIENT);
						} else
							System.out.println("[Immibis's Microblocks] failed to set microblock data at "+x+","+y+","+z+". Tile entity is "+Minecraft.getMinecraft().theWorld.getTileEntity(x, y, z));
				}
			}, Side.CLIENT);
	}

}
